import json
import re
import pytest
from pathlib import Path

import opa_tools.utils
from opa_tools.request.retrieval import get_request, get_request_identifiers
from opa_tools.request.utils import ALLOWED_KINDS

from test_utils import request_dir, monkeysession # type: ignore


def test_get_request_no_flat(request_dir):
    request = get_request(1, kind="*", flat=False, single_as_list=False)
    assert isinstance(request, list)
    assert len(request) > 0
    assert len(request) == len(ALLOWED_KINDS)


def test_get_request_flat(request_dir):
    request = get_request(1, kind="*", flat=True, single_as_list=False)
    assert isinstance(request, dict)
    assert len(request) > 0
    assert all(
        any(
            key.startswith(kind)
            for kind
            in ALLOWED_KINDS
        ) for key in request
    )

@pytest.mark.parametrize("kind", ["gsv", "opa"])
def test_get_request_kind(request_dir, kind):
    request_gsv = get_request(1, kind=kind, flat=True, single_as_list=False)
    request = get_request(1, flat=True, single_as_list=False)
    assert isinstance(request, dict) and isinstance(request_gsv, dict)
    assert len(request) > 0 and len(request_gsv) > 0
    filtered_request = {
        str.removeprefix(key, f"{kind}_"): value
        for key, value
        in request.items()
        if key.startswith(f"{kind}_")
    }
    assert filtered_request == request_gsv

def test_get_identifiers(request_dir):
    all_identifiers = []
    for month_dir in request_dir.iterdir():
        month = month_dir.name
        req_names = [path.name for path in month_dir.iterdir()]
        identifiers = [f"{month}/{req_name}" for req_name in req_names]
        assert set(identifiers) == set(get_request_identifiers(month))
        all_identifiers.extend(identifiers)
    assert set(all_identifiers) == set(get_request_identifiers())

