from datetime import datetime
from pathlib import Path
import pytest
import yaml

from opa_tools.entrypoints.req import req

from test_utils import request_dir, monkeysession # type: ignore
import subprocess

yymm = datetime.now().strftime("%y%m")

def test_req_show(capsys):
    with pytest.raises(NotImplementedError):
        req(["show"])

def test_req_ls(capsys, request_dir):
    req(["ls"])

def test_req_ls_summarized(capsys, request_dir):
    req(["ls", "-s"])
    out, err = capsys.readouterr()
    assert any(line.endswith("*") for line in out.split())

def test_req_col(capsys, request_dir):
    with pytest.raises(NotImplementedError):
        req(["col"])

def test_req_generate(capsys, request_dir: Path):
    new_request = request_dir / yymm / "901" / "grequest.yaml"
    new_request.parent.mkdir(parents=True)
    with open(new_request, "w") as wf:
        yaml.dump({
            "feature_1": [1, 2, 3],
            "feature_2": ["a", "b", "c", "d", "e"],
            "feature_3": [24.8],
        }, wf)
    req(["gen", "901"])
    subrequest_dirs = [path for path in new_request.parent.iterdir() if path.is_dir()]
    assert len(subrequest_dirs) == 15

def test_req_copy(capsys, request_dir):
    with pytest.raises(NotImplementedError):
        req(["cp", "2410/001", "902"])

def test_req_template(capsys, request_dir):
    req(["template", "903", "--from", "2410/001", "2410/002"])
    request_dir /= yymm
    request_dir /= "903"
    assert request_dir.exists()
    assert (request_dir / "grequest.yaml").exists()

